from mythx_models.response.analysis_submission import AnalysisSubmissionResponse


class AnalysisStatusResponse(AnalysisSubmissionResponse):
    """The API response domain model for the status of a single analysis."""

    pass
