"""This module contains the MythX request domain models"""

from mythx_models.request.analysis_list import AnalysisListRequest
from mythx_models.request.analysis_status import AnalysisStatusRequest
from mythx_models.request.analysis_submission import AnalysisSubmissionRequest
from mythx_models.request.auth_login import AuthLoginRequest
from mythx_models.request.auth_logout import AuthLogoutRequest
from mythx_models.request.auth_refresh import AuthRefreshRequest
from mythx_models.request.detected_issues import DetectedIssuesRequest
from mythx_models.request.oas import OASRequest
from mythx_models.request.version import VersionRequest
